@@ -3,26 +3,17 @@
 #
 # unit tests for littletable library
 #
-import types
-import unittest
-import littletable as lt
+from collections import namedtuple
+import io
 import itertools
 import json
-from collections import namedtuple
 from operator import attrgetter
+import sys
 import textwrap
-try:
-    from types import SimpleNamespace
-except ImportError:
-    class SimpleNamespace(object):
-        def __init__(self, **kwargs):
-            self.__dict__.update(kwargs)
-        def __repr__(self):
-            keys = sorted(self.__dict__)
-            items = ("{}={!r}".format(k, self.__dict__[k]) for k in keys)
-            return "{}({})".format(type(self).__name__, ", ".join(items))
-        def __eq__(self, other):
-            return vars(self) == vars(other)
+from types import SimpleNamespace
+import unittest
+
+import littletable as lt
 
 try:
     import dataclasses
@@ -30,31 +21,20 @@ except ImportError:
     # pre Py3.7 (or 3.6 with backported dataclasses)
     dataclasses = None
 else:
-    # must be wrapped in exec, since this syntax is not legal in earlier Pythons
-    exec("""\
-@dataclasses.dataclass
-class DataDataclass:
-    a: int
-    b: int
-    c: int
-""")
+    @dataclasses.dataclass
+    class DataDataclass:
+        a: int
+        b: int
+        c: int
 
 DataTuple = namedtuple("DataTuple", "a b c")
 
-import sys
-PY_2 = sys.version_info[0] == 2
-PY_3 = sys.version_info[0] == 3
-
-if PY_3:
-    import io
-else:
-    import StringIO as io
-
 # if rich is not installed, disable table.present() calls
 try:
     import rich
 except ImportError:
     rich = None
+    # disable present() method, since rich is not available
     lt.Table.present = lambda *args, **kwargs: None
 
 
@@ -124,6 +104,38 @@ class TestTableTypes(unittest.TestCase):
             t.get_index("x")['a'] = 100
 
 
+def announce_test(fn):
+    def _inner(*args):
+        print("\n" + "-" * 50)
+        print(fn.__qualname__)
+        return fn(*args)
+
+    return _inner
+
+
+def make_test_class(*classes):
+
+    class_name = "_".join(c.__name__ for c in classes)
+    if not issubclass(classes[0], unittest.TestCase):
+        cls = type(class_name, (unittest.TestCase, *classes), {})
+    else:
+        cls = type(class_name, tuple(classes), {})
+    for attr in dir(cls):
+        attrvalue = getattr(cls, attr)
+        if attr.startswith("test_") and callable(attrvalue):
+            setattr(cls, attr, announce_test(attrvalue))
+    globals()[cls.__name__] = cls
+
+
+def make_test_classes(cls):
+    make_test_class(cls, UsingDataObjects)
+    make_test_class(cls, UsingNamedtuples)
+    make_test_class(cls, UsingSlottedObjects)
+    make_test_class(cls, UsingSimpleNamespace)
+    if dataclasses is not None:
+        make_test_class(cls, UsingDataclasses)
+
+
 class AbstractContentTypeFactory:
     data_object_type = None
 
@@ -131,29 +143,33 @@ class AbstractContentTypeFactory:
     def make_data_object(cls, a, b, c):
         return cls.data_object_type(a=a, b=b, c=c)
 
+
 class UsingDataObjects(AbstractContentTypeFactory):
     data_object_type = lt.DataObject
 
+
 class UsingNamedtuples(AbstractContentTypeFactory):
     data_object_type = DataTuple
 
+
 class UsingSlottedObjects(AbstractContentTypeFactory):
     data_object_type = Slotted
 
+
 class UsingSimpleNamespace(AbstractContentTypeFactory):
     data_object_type = SimpleNamespace
 
 if dataclasses is not None:
     class UsingDataclasses(AbstractContentTypeFactory):
         data_object_type = DataDataclass
+else:
+    UsingDataclasses = AbstractContentTypeFactory
 
 
 def load_table(table, rec_factory_fn, table_size):
     test_size = table_size
-    for aa in range(test_size):
-        for bb in range(test_size):
-            for cc in range(test_size):
-                table.insert(rec_factory_fn(aa, bb, cc))
+    for aa, bb, cc in itertools.product(range(test_size), repeat=3):
+        table.insert(rec_factory_fn(aa, bb, cc))
 
 
 def make_test_table(rec_factory_fn, table_size):
@@ -166,6 +182,7 @@ def make_dataobject_from_ob(rec):
     return lt.DataObject(**dict((k, getattr(rec, k)) for k in lt._object_attrnames(rec)))
 
 
+@make_test_classes
 class TableCreateTests:
     def test_inserts(self):
         table = lt.Table()
@@ -253,6 +270,79 @@ class TableCreateTests:
         self.assertEqual(test_size*test_size*(test_size-4), len(table.where(a=lt.Table.not_in([2, 4, 6, 8]))))
         self.assertEqual(test_size*test_size*test_size, len(table.where(a=lt.Table.not_in([]))))
 
+        # add a record containing a None value to test is_none and is_not_none comparators
+        table.insert(self.make_data_object(a=1, b=2, c=None))
+        self.assertEqual(1, len(table.where(c=lt.Table.is_none())))
+        self.assertEqual(test_size*test_size*test_size, len(table.where(c=lt.Table.is_not_none())))
+
+        # add a record containing a missing value to test is_null and is_not_null comparators
+        table.insert(self.make_data_object(a=1, b=2, c=""))
+        self.assertEqual(2, len(table.where(c=lt.Table.is_null())))
+        self.assertEqual(test_size * test_size * test_size, len(table.where(c=lt.Table.is_not_null())))
+
+        try:
+            table.insert(self.make_data_object(a=1, b=2))
+        except TypeError:
+            # not all data object types being tested support missing attributes
+            pass
+        else:
+            self.assertEqual(3, len(table.where(c=lt.Table.is_null())))
+            self.assertEqual(test_size * test_size * test_size, len(table.where(c=lt.Table.is_not_null())))
+
+    def test_where_str_comparator(self):
+        unicode_numbers = lt.Table().csv_import(textwrap.dedent("""\
+            name,code_value,numeric_value
+            ROMAN NUMERAL ONE,8544,1
+            ROMAN NUMERAL TWO,8545,2
+            ROMAN NUMERAL THREE,8546,3
+            ROMAN NUMERAL FOUR,8547,4
+            ROMAN NUMERAL FIVE,8548,5
+            ROMAN NUMERAL SIX,8549,6
+            ROMAN NUMERAL SEVEN,8550,7
+            ROMAN NUMERAL EIGHT,8551,8
+            ROMAN NUMERAL NINE,8552,9
+            ROMAN NUMERAL TEN,8553,10
+            SUPERSCRIPT TWO,178,2
+            SUPERSCRIPT THREE,179,3
+            SUPERSCRIPT ONE,185,1
+            SUPERSCRIPT ZERO,8304,0
+            SUPERSCRIPT FOUR,8308,4
+            SUPERSCRIPT FIVE,8309,5
+            SUPERSCRIPT SIX,8310,6
+            SUPERSCRIPT SEVEN,8311,7
+            SUPERSCRIPT EIGHT,8312,8
+            SUPERSCRIPT NINE,8313,9
+            CIRCLED DIGIT ONE,9312,1
+            CIRCLED DIGIT TWO,9313,2
+            CIRCLED DIGIT THREE,9314,3
+            CIRCLED DIGIT FOUR,9315,4
+            CIRCLED DIGIT FIVE,9316,5
+            CIRCLED DIGIT SIX,9317,6
+            CIRCLED DIGIT SEVEN,9318,7
+            CIRCLED DIGIT EIGHT,9319,8
+            CIRCLED DIGIT NINE,9320,9
+            CIRCLED DIGIT ZERO,9450,0
+            """))
+
+        ones = unicode_numbers.where(name=lt.Table.endswith("ONE"))
+        self.assertEqual(3, len(ones))
+
+        supers = unicode_numbers.where(name=lt.Table.startswith("SUPERSCRIPT"))
+        self.assertEqual(10, len(supers))
+
+        # import re
+        # sevens = unicode_numbers.where(lambda rec: re.compile(r".*SEVEN$").match(rec.name))
+
+        sevens = unicode_numbers.where(name=lt.Table.re_match(r".*SEVEN$"))
+        self.assertEqual(3, len(sevens))
+
+        # make names all title case
+        unicode_numbers.add_field("name", lambda rec: rec.name.title())
+        # use regex with re flag
+        import re
+        circled = unicode_numbers.where(name=lt.Table.re_match(r"circled", flags=re.I))
+        self.assertEqual(10, len(circled))
+
     def test_get_slice(self):
         test_size = 10
         table = make_test_table(self.make_data_object, test_size)
@@ -414,23 +504,7 @@ class TableCreateTests:
                          t1_info, "invalid info results")
 
 
-class TableCreateTests_DataObjects(unittest.TestCase, TableCreateTests, UsingDataObjects):
-    pass
-
-class TableCreateTests_Namedtuples(unittest.TestCase, TableCreateTests, UsingNamedtuples):
-    pass
-
-class TableCreateTests_Slotted(unittest.TestCase, TableCreateTests, UsingSlottedObjects):
-    pass
-
-class TableCreateTests_SimpleNamespace(unittest.TestCase, TableCreateTests, UsingSimpleNamespace):
-    pass
-
-if dataclasses is not None:
-    class TableCreateTests_Dataclasses(unittest.TestCase, TableCreateTests, UsingDataclasses):
-        pass
-
-
+@make_test_classes
 class TableListTests:
     def _test_init(self):
         self.test_size = 3
@@ -636,6 +710,9 @@ class TableListTests:
             self.t1.insert(new_rec)
 
         t1_stats = self.t1.stats()
+        t1_stats("t1_stats")
+        t1_stats.csv_export(sys.stdout)
+        t1_stats.present()
         self.assertEqual(self.test_size ** num_fields - 1, t1_stats.by.name["a"].count)
 
     def test_splitby(self):
@@ -666,23 +743,7 @@ class TableListTests:
         print(list(mults_of_3.all.a))
 
 
-class TableListTests_DataObjects(unittest.TestCase, TableListTests, UsingDataObjects):
-    pass
-
-class TableListTests_Namedtuples(unittest.TestCase, TableListTests, UsingNamedtuples):
-    pass
-
-class TableListTests_Slotted(unittest.TestCase, TableListTests, UsingSlottedObjects):
-    pass
-
-class TableListTests_SimpleNamespace(unittest.TestCase, TableListTests, UsingSimpleNamespace):
-    pass
-
-if dataclasses is not None:
-    class TableListTests_Dataclasses(unittest.TestCase, TableListTests, UsingDataclasses):
-        pass
-
-
+@make_test_classes
 class TableJoinTests:
     def test_simple_join(self):
         test_size = 10
@@ -832,23 +893,8 @@ class TableJoinTests:
         print(sorted(full.all.student_id))
         self.assertEqual(['0002', '0004', '0006'], sorted(full.all.student_id))
 
-class TableJoinTests_DataObjects(unittest.TestCase, TableJoinTests, UsingDataObjects):
-    pass
-
-class TableJoinTests_Namedtuples(unittest.TestCase, TableJoinTests, UsingNamedtuples):
-    pass
-
-class TableJoinTests_Slotted(unittest.TestCase, TableJoinTests, UsingSlottedObjects):
-    pass
-
-class TableJoinTests_SimpleNamespace(unittest.TestCase, TableJoinTests, UsingSimpleNamespace):
-    pass
-
-if dataclasses is not None:
-    class TableJoinTests_Dataclasses(unittest.TestCase, TableJoinTests, UsingDataclasses):
-        pass
-
 
+@make_test_classes
 class TableTransformTests:
     def test_sort(self):
         test_size = 10
@@ -954,23 +1000,8 @@ class TableTransformTests:
         t3 = t1.unique(key=lambda rec: rec.c)
         self.assertEqual(test_size, len(t3))
 
-class TableTransformTests_DataObjects(unittest.TestCase, TableTransformTests, UsingDataObjects):
-    pass
-
-class TableTransformTests_Namedtuples(unittest.TestCase, TableTransformTests, UsingNamedtuples):
-    pass
-
-class TableTransformTests_Slotted(unittest.TestCase, TableTransformTests, UsingSlottedObjects):
-    pass
-
-class TableTransformTests_SimpleNamespace(unittest.TestCase, TableTransformTests, UsingSimpleNamespace):
-    pass
-
-if dataclasses is not None:
-    class TableTransformTests_Dataclasses(unittest.TestCase, TableTransformTests, UsingDataclasses):
-        pass
-
 
+@make_test_classes
 class TableOutputTests:
     def test_basic_present(self):
         if rich is None:
@@ -981,60 +1012,77 @@ class TableOutputTests:
         from rich import box
         from io import StringIO
         table = lt.Table().csv_import(textwrap.dedent("""\
-        a,b
-        10,100
-        20,200
-        """))
+            a,b
+            10,100
+            20,200
+            """))
         table.present()
         out = StringIO()
         table.present(file=out, box=box.ASCII)
         expected = textwrap.dedent("""\
-        +----------+
-        | A  | B   |
-        |----+-----|
-        | 10 | 100 |
-        | 20 | 200 |
-        +----------+
-        """)
+            +----------+
+            | A  | B   |
+            |----+-----|
+            | 10 | 100 |
+            | 20 | 200 |
+            +----------+
+            """)
         self.assertEqual(expected, out.getvalue())
 
         # test bugfix when table has attribute "default"
         table = lt.Table().csv_import(textwrap.dedent("""\
-        a,b,default
-        10,100,purple
-        15,150,
-        20,200,orange
-        """))
+            a,b,default
+            10,100,purple
+            15,150,
+            20,200,orange
+            """))
         table.present()
         out = StringIO()
         table.present(file=out, box=box.ASCII)
         expected = textwrap.dedent("""\
-        +--------------------+
-        | A  | B   | Default |
-        |----+-----+---------|
-        | 10 | 100 | purple  |
-        | 15 | 150 |         |
-        | 20 | 200 | orange  |
-        +--------------------+
-        """)
+            +--------------------+
+            | A  | B   | Default |
+            |----+-----+---------|
+            | 10 | 100 | purple  |
+            | 15 | 150 |         |
+            | 20 | 200 | orange  |
+            +--------------------+
+            """)
         self.assertEqual(expected, out.getvalue())
 
+    def test_markdown(self):
+        table = lt.Table().csv_import(textwrap.dedent("""\
+            a,b
+            10,100
+            20,200
+            """))
+        out = table.as_markdown()
+        print(out)
+        expected = textwrap.dedent("""\
+            | a | b |
+            |---|---|
+            | 10 | 100 |
+            | 20 | 200 |
+            """)
+        self.assertEqual(expected, out)
 
-class TableOutputTests_DataObjects(unittest.TestCase, TableOutputTests, UsingDataObjects):
-    pass
-
-class TableOutputTests_Namedtuples(unittest.TestCase, TableOutputTests, UsingNamedtuples):
-    pass
-
-class TableOutputTests_Slotted(unittest.TestCase, TableOutputTests, UsingSlottedObjects):
-    pass
-
-class TableOutputTests_SimpleNamespace(unittest.TestCase, TableOutputTests, UsingSimpleNamespace):
-    pass
-
-if dataclasses is not None:
-    class TableOutputTests_Dataclasses(unittest.TestCase, TableOutputTests, UsingDataclasses):
-        pass
+        # test bugfix when table has attribute "default"
+        table = lt.Table().csv_import(textwrap.dedent("""\
+            a,b,default
+            10,100,purple
+            15,150,
+            20,200,orange
+            """))
+        out = table.as_markdown()
+        print(out)
+        expected = textwrap.dedent("""\
+            | a | b | default |
+            |---|---|---|
+            | 10 | 100 | purple |
+            | 15 | 150 |  |
+            | 20 | 200 | orange |
+            """)
+        self.assertEqual(expected, out)
 
 
 # sample import data sets
@@ -1132,6 +1180,8 @@ fixed_width_data = """\
 
 """
 
+
+@make_test_classes
 class TableImportExportTests:
     def test_csv_export(self):
         from itertools import permutations
@@ -1198,9 +1248,11 @@ class TableImportExportTests:
         tt = lt.Table().csv_import("test/abc.csv", transforms=dict.fromkeys("abc", int))
         print("abc.csv", tt.info())
 
-        compressed_files = ["abc.csv.zip", "abc.csv.gz"]
-        if PY_3:
-            compressed_files.append("abc.csv.xz")
+        compressed_files = [
+            "abc.csv.zip",
+            "abc.csv.gz",
+            "abc.csv.xz",
+        ]
         for name in compressed_files:
             tt2 = lt.Table().csv_import("test/" + name, transforms=dict.fromkeys("abc", int))
             print(name, tt2.info())
@@ -1209,11 +1261,6 @@ class TableImportExportTests:
             self.assertEqual(sum(tt.all.b), sum(tt2.all.b))
             self.assertEqual(sum(tt.all.c), sum(tt2.all.c))
 
-        if PY_2:
-            with self.assertRaises(Exception):
-                name = "abc.csv.xz"
-                tt2 = lt.Table().csv_import("test/" + name, transforms=dict.fromkeys("abc", int))
-
         tt2 = lt.Table().json_import("test/abc.json.gz")
         print("abc.json.gz", tt2.info())
         self.assertEqual(tt.info(), tt2.info())
@@ -1241,6 +1288,105 @@ class TableImportExportTests:
         print(tt.info())
         self.assertEqual(test_size * test_size, len(tt))
 
+        # test all special comparators when used as filters
+        #     is_none - attribute value is None
+        #     is_not_none - attribute value is not None
+        #     is_null - attribute value is None, "", or not defined
+        #     is_not_null - attribute value is defined, and is not None or ""
+        #     startswith - attribute value starts with a given string
+        #     endswith - attribute value ends with a given string
+        #     re_match - attribute value matches a regular expression
+
+        print()
+        input_data = textwrap.dedent("""\
+        name,a,b,c
+        "A",100,100,100
+        "B",200,,200
+        "A1",101,101,101
+        "B1",201,,201
+        "C1",301,,301
+        ,99,99,99
+        """)
+        lt.Table().csv_import(input_data,
+                              transforms=dict.fromkeys("abc", int),
+                              ).present()
+
+        """
+        +-------------------------+
+        | Name |   A |    B |   C |
+        |------+-----+------+-----|
+        | A    | 100 |  100 | 100 |
+        | B    | 200 | None | 200 |
+        | A1   | 101 |  101 | 101 |
+        | B1   | 201 | None | 201 |
+        | C1   | 301 | None | 301 |
+        |      |  99 |   99 |  99 |
+        +-------------------------+
+        """
+
+        print("is_none()")
+        x = lt.Table().csv_import(input_data,
+                                         transforms=dict.fromkeys("abc", int),
+                                         filters={"b": lt.Table.is_none()})
+        self.assertEqual(3, len(x))
+        self.assertTrue(all(b is None for b in x.all.b))
+
+        print("is_not_none()")
+        x = lt.Table().csv_import(input_data,
+                                  transforms=dict.fromkeys("abc", int),
+                                  filters={"b": lt.Table.is_not_none()})
+        self.assertEqual(3, len(x))
+        self.assertEqual(300, sum(x.all.b))
+
+        print("b is_null()")
+        x = lt.Table().csv_import(input_data,
+                                  transforms=dict.fromkeys("abc", int),
+                                  filters={"b": lt.Table.is_null()})
+        self.assertEqual(3, len(x))
+        self.assertTrue(all(b is None for b in x.all.b))
+
+        print("b is_not_null()")
+        x = lt.Table().csv_import(input_data,
+                                  transforms=dict.fromkeys("abc", int),
+                                  filters={"b": lt.Table.is_not_null()})
+        self.assertEqual(3, len(x))
+        self.assertEqual(300, sum(x.all.b))
+
+        print("name is_null()")
+        x = lt.Table().csv_import(input_data,
+                                  transforms=dict.fromkeys("abc", int),
+                                  filters={"name": lt.Table.is_null()})
+        self.assertEqual(1, len(x))
+        self.assertEqual(3*99, x[0].a + x[0].b + x[0].c)
+
+        print("name is_not_null()")
+        x = lt.Table().csv_import(input_data,
+                                  transforms=dict.fromkeys("abc", int),
+                                  filters={"name": lt.Table.is_not_null()})
+        self.assertEqual(5, len(x))
+        self.assertEqual("A B A1 B1 C1".split(), list(x.all.name))
+
+        print("name startswith('B')")
+        x = lt.Table().csv_import(input_data,
+                                  transforms=dict.fromkeys("abc", int),
+                                  filters={"name": lt.Table.startswith("B")})
+        self.assertEqual(2, len(x))
+        self.assertEqual("B B1".split(), list(x.all.name))
+
+        print("name endswith('1')")
+        x = lt.Table().csv_import(input_data,
+                                  transforms=dict.fromkeys("abc", int),
+                                  filters={"name": lt.Table.endswith("1")})
+        self.assertEqual(3, len(x))
+        self.assertEqual("A1 B1 C1".split(), list(x.all.name))
+
+        print(r"name re_match(r'[AB]\d')")
+        x = lt.Table().csv_import(input_data,
+                                  transforms=dict.fromkeys("abc", int),
+                                  filters={"name": lt.Table.re_match(r"[AB]\d")})
+        self.assertEqual(2, len(x))
+        self.assertEqual("A1 B1".split(), list(x.all.name))
+
     def test_csv_string_import(self):
         data = csv_data
         csvtable = lt.Table().csv_import(csv_source=data, transforms={'a': int, 'b': int, 'c': int})
@@ -1373,23 +1519,8 @@ class TableImportExportTests:
         self.assertTrue(all(make_dataobject_from_ob(rec1) == rec2 for rec1, rec2 in zip(t1, tt)))
         self.assertEqual(len([d for d in data.splitlines() if d.strip()]), len(tt))
 
-class TableImportExportTests_DataObjects(unittest.TestCase, TableImportExportTests, UsingDataObjects):
-    pass
-
-class TableImportExportTests_Namedtuples(unittest.TestCase, TableImportExportTests, UsingNamedtuples):
-    pass
-
-class TableImportExportTests_Slotted(unittest.TestCase, TableImportExportTests, UsingSlottedObjects):
-    pass
-
-class TableImportExportTests_SimpleNamespace(unittest.TestCase, TableImportExportTests, UsingSimpleNamespace):
-    pass
-
-if dataclasses is not None:
-    class TableImportExportTests_Dataclasses(unittest.TestCase, TableImportExportTests, UsingDataclasses):
-        pass
-
 
+@make_test_classes
 class TablePivotTests:
     def test_pivot(self):
         test_size = 5
@@ -1408,22 +1539,6 @@ class TablePivotTests:
 
         # TODO - add asserts
 
-class TablePivotTests_DataObjects(unittest.TestCase, TablePivotTests, UsingDataObjects):
-    pass
-
-class TablePivotTests_Namedtuples(unittest.TestCase, TablePivotTests, UsingNamedtuples):
-    pass
-
-class TablePivotTests_Slotted(unittest.TestCase, TablePivotTests, UsingSlottedObjects):
-    pass
-
-class TablePivotTests_SimpleNamespace(unittest.TestCase, TablePivotTests, UsingSimpleNamespace):
-    pass
-
-if dataclasses is not None:
-    class TablePivotTests_Dataclasses(unittest.TestCase, TablePivotTests, UsingDataclasses):
-        pass
-
 
 class TableSearchTests(unittest.TestCase):
     recipe_data = textwrap.dedent("""\
@@ -1441,19 +1556,22 @@ class TableSearchTests(unittest.TestCase):
         11,Hamburger,ground beef bun lettuce ketchup mustard pickle
         12,Cheeseburger,ground beef bun lettuce ketchup mustard pickle cheese
         13,Bacon cheeseburger,ground beef bun lettuce ketchup mustard pickle cheese bacon
-        """).splitlines()
+        """)
 
     def setUp(self):
         self.recipes = lt.Table().csv_import(self.recipe_data, transforms=dict(id=int))
         self.recipes.create_search_index("ingredients")
 
+    @announce_test
     def test_access_non_existent_search_attribute(self):
         with self.assertRaises(ValueError, msg="failed to raise ValueError when accessing non-existent search index"):
             self.recipes.search.title("xyz")
 
+    @announce_test
     def test_search_dir(self):
         self.assertEqual(['ingredients'], dir(self.recipes.search), "failed to generate correct dir() response")
 
+    @announce_test
     def test_text_search(self):
         for query, expected in [
             ("", []),
@@ -1474,12 +1592,14 @@ class TableSearchTests(unittest.TestCase):
             self.assertEqual(expected, match_ids,
                              "invalid results for query {!r}, expected {}, got {}".format(query, expected, match_ids))
 
+    @announce_test
     def test_invalidate_index(self):
         self.recipes.pop(0)
         with self.assertRaises(lt.SearchIndexInconsistentError,
                                msg="failed to raise exception when searching modified table"):
             self.recipes.search.ingredients("bacon")
 
+    @announce_test
     def test_search_with_keywords(self):
         for query, expected, expected_words in [
                 ("tuna", [1, 6], [{'tuna', 'noodles', 'cream', 'of', 'mushroom', 'soup'},
@@ -1496,6 +1616,7 @@ class TableSearchTests(unittest.TestCase):
                                                                                               expected_words,
                                                                                               match_words))
 
+    @announce_test
     def test_search_with_limit(self):
         for query, expected in [
             ("", []),
@@ -1516,6 +1637,7 @@ class TableSearchTests(unittest.TestCase):
             self.assertEqual(expected, match_ids,
                              "invalid results for query {!r}, expected {}, got {}".format(query, expected, match_ids))
 
+    @announce_test
     def test_search_with_min_score(self):
         for query, expected in [
             ("", []),
@@ -1555,10 +1677,6 @@ if dataclasses is not None:
 
 
 class InitialTest(unittest.TestCase):
-    if sys.version_info[:2] <= (2, 6):
-        print('unit_tests.py only runs on Python 2.7 or later')
-        sys.exit(0)
-
     from littletable import (
         __version__ as littletable_version,
         __version_time__ as littletable_version_time,
@@ -1566,9 +1684,7 @@ class InitialTest(unittest.TestCase):
     )
 
     print(
-        "Beginning test of littletable, version",
-        littletable_version,
-        littletable_version_time,
+        f"Beginning test of littletable, version {littletable_version}, {littletable_version_time}",
     )
     print(littletable_version_info)
     print("Python version", sys.version)
